#ifndef MSP_STRINGCODEC_ISO2022JP_H_
#define MSP_STRINGCODEC_ISO2022JP_H_

#include "codec.h"

namespace Msp {
namespace StringCodec {

class Iso2022Jp: public StandardCodec<Iso2022Jp>
{
public:
	enum Mode
	{
		ASCII,
		JISX0201,
		JISX0208
	};

	class Encoder: public Codec::Encoder
	{
	private:
		Mode mode = ASCII;

	public:
		Encoder(ErrorMode em = DEFAULT): Codec::Encoder(em) { }

		void encode_char(unichar, std::string &) override;
		void sync(std::string &) override;
		void reset() override;
	private:
		void switch_mode(Mode, std::string &);
		void transliterate(unichar, std::string &) override;
	};

	class Decoder: public Codec::Decoder
	{
	private:
		Mode mode = ASCII;
		Codec::Decoder *dec = nullptr;

	public:
		Decoder(ErrorMode = DEFAULT);

		unichar decode_char(const std::string &, std::string::const_iterator &) override;
		void reset() override;
	private:
		void switch_mode(Mode);
	};

	Iso2022Jp(ErrorMode em = DEFAULT): StandardCodec<Iso2022Jp>(em) { }

	const char *get_name() const override { return "ISO-2022-JP"; }
};

} // namespace StringCodec
} // namespace Msp

#endif
